"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FetchHttpHandler = void 0;
const communicationProxy_1 = require("./utils/communicationProxy");
const command_1 = require("./types/command");
const buildAmzTarget_1 = require("./utils/buildAmzTarget");
class FetchHttpHandler {
    constructor(config) {
        this.config = config !== null && config !== void 0 ? config : {};
        (0, communicationProxy_1.subscribeToChannel)(this.channelRequestHandler.bind(this));
    }
    setRuntimeConfig(config) {
        this.runtimeConfig = config;
    }
    async responseHandler(response) {
        const { status, statusText, ok, headers, body } = response;
        const fetchHeaders = headers;
        const transformedHeaders = {};
        for (const pair of fetchHeaders.entries()) {
            transformedHeaders[pair[0]] = pair[1];
        }
        const hasReadableStream = body !== undefined;
        if (!hasReadableStream) {
            return response.blob()
                .then(() => {
                return {
                    status,
                    statusText,
                    ok,
                    headers: transformedHeaders,
                    body: response.json(),
                };
            });
        }
        const reader = body.getReader();
        let res = new Uint8Array(0);
        let isDone = false;
        while (!isDone) {
            const { done, value } = await reader.read();
            if (value) {
                const prior = res;
                res = new Uint8Array(prior.length + value.length);
                res.set(prior);
                res.set(value, prior.length);
            }
            isDone = done;
        }
        return {
            status,
            statusText,
            ok,
            headers: transformedHeaders,
            body: JSON.parse(new TextDecoder('utf8').decode(res)),
        };
    }
    async channelRequestHandler(_, options) {
        try {
            const { headers, body } = options;
            const amzTarget = headers === null || headers === void 0 ? void 0 : headers['x-amz-target'];
            const clientMethod = (0, buildAmzTarget_1.parseAmzTarget)(amzTarget);
            const Command = command_1.Commands[clientMethod];
            const clientCommand = new Command(JSON.parse(body));
            return this.handle(clientCommand.serialize(this.runtimeConfig), {});
        }
        catch (e) {
            console.log('Something went wrong during request.', e);
            return Promise.reject(e);
        }
    }
    async fetchRequestHandler(url, options) {
        try {
            const response = await fetch(url, options);
            return this.responseHandler(response);
        }
        catch (e) {
            console.log('Something went wrong during request.', e);
            return Promise.reject(e);
        }
    }
    async fetchRequest(url, options, frameWindow) {
        try {
            if (frameWindow) {
                const response = await (0, communicationProxy_1.fetchWithChannel)(frameWindow.contentWindow, frameWindow.src, { url, options });
                return Promise.resolve(response);
            }
            else {
                return this.fetchRequestHandler(url, options);
            }
        }
        catch (e) {
            console.log('Something went wrong during request.', e);
            return Promise.reject(e);
        }
    }
    requestTimeout(timeoutInMs) {
        return new Promise((_, reject) => {
            if (timeoutInMs) {
                setTimeout(() => {
                    const timeoutError = new Error(`Request did not complete within ${timeoutInMs} ms.`);
                    timeoutError.name = "TimeoutError";
                    reject(timeoutError);
                }, timeoutInMs);
            }
        });
    }
    abortRequest(abortSignal) {
        return new Promise((_, reject) => {
            if (abortSignal) {
                abortSignal.onabort = () => {
                    const abortError = new Error('Request aborted.');
                    abortError.name = 'AbortError';
                    reject(abortError);
                };
            }
        });
    }
    handle(request, options) {
        const { abortSignal } = options || {};
        const { requestTimeout } = this.config || {};
        if (abortSignal === null || abortSignal === void 0 ? void 0 : abortSignal.aborted) {
            const abortError = new Error("Request aborted");
            abortError.name = "AbortError";
            return Promise.reject(abortError);
        }
        const { protocol, hostname, port, path, method, headers, body, frameWindow } = request;
        const url = `${protocol}//${hostname}${port ? `:${port}` : ''}${path}`;
        const requestOptions = {
            method,
            headers,
            body,
            ...(!frameWindow && AbortController && abortSignal && { signal: abortSignal }),
        };
        return Promise.race([
            this.fetchRequest(url, requestOptions, frameWindow),
            this.requestTimeout(requestTimeout),
            this.abortRequest(abortSignal),
        ]);
    }
}
exports.FetchHttpHandler = FetchHttpHandler;
