"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const worker_threads_1 = require("worker_threads");
const wisdomClient_1 = require("./wisdomClient");
const getAuthorizedWidgetsForUser_1 = require("./commands/getAuthorizedWidgetsForUser");
const getContent_1 = require("./commands/getContent");
const getRecommendations_1 = require("./commands/getRecommendations");
const listIntegrationAssociations_1 = require("./commands/listIntegrationAssociations");
const notifyRecommendationsReceived_1 = require("./commands/notifyRecommendationsReceived");
const queryAssistant_1 = require("./commands/queryAssistant");
const searchSessions_1 = require("./commands/searchSessions");
const getContact_1 = require("./commands/getContact");
describe('WisdomClient', () => {
    let client;
    const instanceUrl = 'https://foo.amazonaws.com';
    const mockHandler = jest.fn((args) => Promise.resolve('success response'));
    const mockResolveRequestHandler = jest.fn((args) => mockHandler);
    delete global.location;
    global.location = { href: 'https://foo.amazonaws.com' };
    global.MessageChannel = worker_threads_1.MessageChannel;
    beforeEach(() => {
        jest.clearAllMocks();
        client = new wisdomClient_1.WisdomClient({
            instanceUrl,
        });
    });
    it('should properly construct the endpoint from the instanceUrl when not provided', () => {
        client = new wisdomClient_1.WisdomClient({
            instanceUrl,
            endpoint: 'https://localhost:4000/example-api/',
        });
        expect(client.config.endpoint).toEqual('https://localhost:4000/example-api/');
        const cfInstanceUrl = 'https://foo.com.awsapps.com/connect/';
        client = new wisdomClient_1.WisdomClient({
            instanceUrl: cfInstanceUrl,
        });
        expect(client.config.endpoint).toEqual('https://foo.com.awsapps.com/connect/agent-app/api');
        const nginxInstanceUrl = 'https://foo.my.connect.aws';
        client = new wisdomClient_1.WisdomClient({
            instanceUrl: nginxInstanceUrl,
        });
        expect(client.config.endpoint).toEqual('https://foo.my.connect.aws/agent-app/api');
    });
    it('should return a response promise when calling getAuthorizedWidgetsForUser', async () => {
        getAuthorizedWidgetsForUser_1.GetAuthorizedWidgetsForUser.prototype.resolveRequestHandler = mockResolveRequestHandler;
        await expect(client.getAuthorizedWidgetsForUser({})).resolves.toEqual('success response');
    });
    it('should return a response promise when calling getContent', async () => {
        getContent_1.GetContent.prototype.resolveRequestHandler = mockResolveRequestHandler;
        await expect(client.getContent({
            contentId: 'c6ca58b9-b046-4664-ba63-da17f51fc332',
            knowledgeBaseId: 'f9b5fa90-b3ce-45c9-9967-582c87074864',
        })).resolves.toEqual('success response');
    });
    it('should return a response promise when calling getRecommendations', async () => {
        getRecommendations_1.GetRecommendations.prototype.resolveRequestHandler = mockResolveRequestHandler;
        await expect(client.getRecommendations({
            assistantId: 'b5b0e4af-026e-4472-9371-d171a9fdf75a',
            sessionId: '249bbb30-aede-42a8-be85-d8483c317686',
        })).resolves.toEqual('success response');
    });
    it('should return a response promise when calling listIntegrationAssociations', async () => {
        listIntegrationAssociations_1.ListIntegrationAssociations.prototype.resolveRequestHandler = mockResolveRequestHandler;
        await expect(client.listIntegrationAssociations({
            InstanceId: 'b5b0e4af-026e-4472-9371-d171a9fdf75a',
        })).resolves.toEqual('success response');
    });
    it('should return a response promise when calling notifyRecommendationsReceived', async () => {
        notifyRecommendationsReceived_1.NotifyRecommendationsReceived.prototype.resolveRequestHandler = mockResolveRequestHandler;
        await expect(client.notifyRecommendationsReceived({
            assistantId: 'b5b0e4af-026e-4472-9371-d171a9fdf75a',
            sessionId: '249bbb30-aede-42a8-be85-d8483c317686',
            recommendationIds: [],
        })).resolves.toEqual('success response');
    });
    it('should return a response promise when calling queryAssistant', async () => {
        queryAssistant_1.QueryAssistant.prototype.resolveRequestHandler = mockResolveRequestHandler;
        await expect(client.queryAssistant({
            assistantId: 'b5b0e4af-026e-4472-9371-d171a9fdf75a',
            maxResults: 10,
            queryText: 'aws',
        })).resolves.toEqual('success response');
    });
    it('should return a response promise when calling searchSessions', async () => {
        searchSessions_1.SearchSessions.prototype.resolveRequestHandler = mockResolveRequestHandler;
        await expect(client.searchSessions({
            assistantId: 'b5b0e4af-026e-4472-9371-d171a9fdf75a',
            searchExpression: {
                filters: [
                    {
                        operator: 'equals',
                        field: 'name',
                        value: '249bbb30-aede-42a8-be85-d8483c317686',
                    },
                ],
            },
        })).resolves.toEqual('success response');
    });
    it('should return a response promise when calling getContact', async () => {
        getContact_1.GetContact.prototype.resolveRequestHandler = mockResolveRequestHandler;
        await expect(client.getContact({
            awsAccountId: '85be2a14-94d7-41f2-835e-85368acb55df',
            instanceId: 'b5b0e4af-026e-4472-9371-d171a9fdf75a',
            contactId: '249bbb30-aede-42a8-be85-d8483c317686',
        })).resolves.toEqual('success response');
    });
});
