import { __awaiter, __generator } from "tslib";
import { FetchHttpHandler } from './fetchHttpHandler';
import { HttpRequest } from './httpRequest';
import { getRuntimeConfig } from './utils/runtimeConfig.shared';
describe('FetchHttpHandler', function () {
    var mockRequest = new HttpRequest({
        headers: {},
        hostname: 'foo.amazonaws.com',
        method: 'POST',
        path: "/test/?example=param",
        protocol: "https",
    });
    var mockResponse = {
        headers: {
            entries: jest.fn().mockReturnValue([
                ['foo', 'bar'],
            ]),
        },
        blob: jest.fn(function () { return Promise.resolve('success response'); }),
        json: jest.fn().mockReturnValue('success response'),
    };
    var mockAbortSignal = { aborted: false, onabort: null };
    var mockAbort = jest.fn(function () { return mockAbortSignal.aborted = true; });
    var mockAbortController = function () { return ({
        abort: mockAbort,
        signal: mockAbortSignal,
    }); };
    var mockFetch = jest.fn().mockResolvedValue(mockResponse);
    var mockMessageChannel = jest.fn().mockReturnValue({
        port1: {
            onmessage: jest.fn(),
        },
        port2: {
            onmessage: jest.fn(),
        },
    });
    var fetchHttpHandler = new FetchHttpHandler();
    fetchHttpHandler.setRuntimeConfig(getRuntimeConfig({
        instanceUrl: 'https://foo.amazonaws.com',
    }));
    beforeEach(function () {
        jest.clearAllMocks();
        global.AbortController = jest.fn().mockReturnValue(mockAbortController);
        global.fetch = mockFetch;
        global.MessageChannel = mockMessageChannel;
    });
    it('should make requests using fetch if no frame window provided', function () { return __awaiter(void 0, void 0, void 0, function () {
        var response;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, fetchHttpHandler.handle({}, {})];
                case 1:
                    response = _a.sent();
                    expect(mockFetch.mock.calls.length).toEqual(1);
                    expect(response.body).toEqual(mockResponse.json());
                    return [2];
            }
        });
    }); });
    it('should make requests using message channel if frame window provided', function () {
        fetchHttpHandler.handle({
            frameWindow: {
                contentWindow: global,
                src: 'https://foo.amazonaws.com/wisdom-v2',
            },
        }, {});
        expect(mockMessageChannel.mock.calls.length).toEqual(1);
    });
    it('should properly construct the url', function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, fetchHttpHandler.handle(mockRequest, {})];
                case 1:
                    _a.sent();
                    expect(mockFetch.mock.calls.length).toEqual(1);
                    expect(mockFetch.mock.calls[0][0]).toEqual('https://foo.amazonaws.com/test/?example=param');
                    mockRequest = new HttpRequest({
                        port: 4000,
                    });
                    return [4, fetchHttpHandler.handle(mockRequest, {})];
                case 2:
                    _a.sent();
                    expect(mockFetch.mock.calls.length).toEqual(2);
                    expect(mockFetch.mock.calls[1][0]).toEqual('https://localhost:4000/agent-app/api');
                    return [2];
            }
        });
    }); });
    it('should not make requests if already aborted', function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, expect(fetchHttpHandler.handle({}, {
                        abortSignal: {
                            aborted: true,
                            onabort: null,
                        },
                    })).rejects.toHaveProperty('name', 'AbortError')];
                case 1:
                    _a.sent();
                    expect(mockFetch.mock.calls.length).toBe(0);
                    return [2];
            }
        });
    }); });
    it('should pass an abortSignal if supported', function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, fetchHttpHandler.handle({}, {
                        abortSignal: {
                            aborted: false,
                            onabort: null,
                        },
                    })];
                case 1:
                    _a.sent();
                    expect(mockFetch.mock.calls.length).toBe(1);
                    expect(mockFetch.mock.calls[0][1]).toHaveProperty('signal');
                    return [2];
            }
        });
    }); });
    it('should pass a timeout from a provider to requestTimeout method', function () { return __awaiter(void 0, void 0, void 0, function () {
        var fetchHttpHandler, requestTimeoutMock;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    fetchHttpHandler = new FetchHttpHandler({
                        requestTimeout: 500,
                    });
                    requestTimeoutMock = jest.spyOn(fetchHttpHandler, 'requestTimeout');
                    return [4, fetchHttpHandler.handle({}, {})];
                case 1:
                    _a.sent();
                    expect(mockFetch.mock.calls.length).toBe(1);
                    expect(requestTimeoutMock.mock.calls[0][0]).toBe(500);
                    return [2];
            }
        });
    }); });
    it('should throw a timeout error if it times out before the request finishes', function () { return __awaiter(void 0, void 0, void 0, function () {
        var mockFetch, fetchHttpHandler;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    mockFetch = jest.fn(function () { return (new Promise(function () { return null; })); });
                    global.fetch = mockFetch;
                    fetchHttpHandler = new FetchHttpHandler({
                        requestTimeout: 5,
                    });
                    return [4, expect(fetchHttpHandler.handle({}, {})).rejects.toHaveProperty('name', 'TimeoutError')];
                case 1:
                    _a.sent();
                    expect(mockFetch.mock.calls.length).toBe(1);
                    return [2];
            }
        });
    }); });
    it('should throw an unsupported service prefix error if the incoming MessageChannel request x-amz-target is invalid', function () { return __awaiter(void 0, void 0, void 0, function () {
        var requestOptions;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    requestOptions = {
                        headers: {
                            'x-amz-target': 'some-client-method',
                        },
                    };
                    return [4, expect(fetchHttpHandler.channelRequestHandler('', requestOptions)).rejects.toThrowError('Unsupported service prefix.')];
                case 1:
                    _a.sent();
                    return [2];
            }
        });
    }); });
    it('should throw an unsupported service error if the incoming MessageChannel request x-amz-target is invalid', function () { return __awaiter(void 0, void 0, void 0, function () {
        var requestOptions;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    requestOptions = {
                        headers: {
                            'x-amz-target': 'AgentAppService.Cases.some-client-method',
                        },
                    };
                    return [4, expect(fetchHttpHandler.channelRequestHandler('', requestOptions)).rejects.toThrowError('Unsupported service.')];
                case 1:
                    _a.sent();
                    return [2];
            }
        });
    }); });
    it('should throw an unsupported client method error if the incoming MessageChannel request x-amz-target is invalid', function () { return __awaiter(void 0, void 0, void 0, function () {
        var requestOptions;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    requestOptions = {
                        headers: {
                            'x-amz-target': 'AgentAppService.WisdomV2.some-client-method',
                        },
                    };
                    return [4, expect(fetchHttpHandler.channelRequestHandler('', requestOptions)).rejects.toThrowError('Unsupported client method.')];
                case 1:
                    _a.sent();
                    return [2];
            }
        });
    }); });
    it('should throw an invalid user input error if the incoming MessageChannel request body is invalid', function () { return __awaiter(void 0, void 0, void 0, function () {
        var requestUrl, requestOptions;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    requestUrl = 'https://other-origin-domain.amazonaws.com';
                    requestOptions = {
                        headers: {
                            'x-amz-target': 'AgentAppService.Acs.listIntegrationAssociations',
                        },
                        body: JSON.stringify({}),
                    };
                    return [4, expect(fetchHttpHandler.channelRequestHandler(requestUrl, requestOptions)).rejects.toThrowError('Invalid InstanceId.')];
                case 1:
                    _a.sent();
                    requestOptions = {
                        headers: {
                            'x-amz-target': 'AgentAppService.Lcms.getContact',
                        },
                        body: JSON.stringify({
                            instanceId: '1234567890',
                        }),
                    };
                    return [4, expect(fetchHttpHandler.channelRequestHandler(requestUrl, requestOptions)).rejects.toThrowError('Invalid awsAccountId.')];
                case 2:
                    _a.sent();
                    return [2];
            }
        });
    }); });
    it('should reconstruct the url even if the incoming MessageChannel request body is valid', function () { return __awaiter(void 0, void 0, void 0, function () {
        var requestUrl, requestOptions;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    requestUrl = 'https://other-origin-domain.amazonaws.com';
                    requestOptions = {
                        headers: {
                            'x-amz-target': 'AgentAppService.Acs.listIntegrationAssociations',
                        },
                        body: JSON.stringify({
                            InstanceId: '1234567890',
                        }),
                    };
                    return [4, fetchHttpHandler.channelRequestHandler(requestUrl, requestOptions)];
                case 1:
                    _a.sent();
                    expect(mockFetch.mock.calls.pop()).toEqual([
                        'https://foo.amazonaws.com/agent-app/api',
                        {
                            method: 'POST',
                            headers: {
                                'x-amz-target': 'AgentAppService.Acs.listIntegrationAssociations'
                            },
                            body: JSON.stringify({
                                InstanceId: '1234567890',
                            }),
                        },
                    ]);
                    return [2];
            }
        });
    }); });
});
